package myproject;

import jetbrains.mps.logging.Logger;
import jetbrains.mps.project.GlobalScope;
import jetbrains.mps.project.IModule;
import jetbrains.mps.project.SModelRoot;
import jetbrains.mps.smodel.*;
import jetbrains.mps.smodel.persistence.AbstractModelRootManager;
import jetbrains.mps.smodel.persistence.IModelRootManager;
import jetbrains.mps.vfs.FileSystem;
import jetbrains.mps.vfs.IFile;

import java.util.HashSet;
import java.util.Set;


public class ModelRootManager2 extends AbstractModelRootManager {

    private static final Logger LOG = Logger.getLogger(ModelRootManager2.class);

    
    @org.jetbrains.annotations.NotNull
    public Set<SModelDescriptor> read(
        @org.jetbrains.annotations.NotNull SModelRoot root,
        @org.jetbrains.annotations.NotNull IModule owner) {

        final HashSet<SModelDescriptor> result = new HashSet<SModelDescriptor>();
        readModelDescriptors(result, FileSystem.getFile(root.getPath()), root, owner);
        return result;
    }


    private void readModelDescriptors(HashSet<SModelDescriptor> result, IFile dir, SModelRoot root, IModule owner) {

        final IFile hardcoded = FileSystem.getFile("C:\\temp");

//        SModelReference modelReference = PathManager.getModelUID(hardcoded, hardcoded, root.getPrefix());
        SModelReference modelReference = new SModelReference("persons", "");

        SModelDescriptor modelDescriptor = getInstance(this, hardcoded.getAbsolutePath(), modelReference, owner, false);


        result.add(modelDescriptor);
    }


    private static SModelDescriptor getInstance(IModelRootManager manager, String fileName, SModelReference modelReference, ModelOwner owner, boolean fireModelCreated) {


        SModelRepository modelRepository = SModelRepository.getInstance();
        SModelDescriptor modelDescriptor = modelRepository.getModelDescriptor(modelReference);
        if (modelDescriptor != null) {
            modelRepository.addOwnerForDescriptor(modelDescriptor, owner);
            return modelDescriptor;
        } else {
            IFile modelFile = FileSystem.getFile(fileName);
            modelDescriptor = new DefaultSModelDescriptor(manager, modelFile, modelReference);
            if (fireModelCreated) {
                modelRepository.createNewModel(modelDescriptor, owner);
            } else {
                modelRepository.registerModelDescriptor(modelDescriptor, owner);
            }
            return modelDescriptor;
        }
    }


    @org.jetbrains.annotations.NotNull
    public SModel loadModel(@org.jetbrains.annotations.NotNull SModelDescriptor modelDescriptor) {

        final SModel sModel = new SModel(modelDescriptor.getSModelReference());

        sModel.setLoading(true);


        final BaseAdapter person1 = newPerson(sModel);
        person1.setProperty("name", "John");

        final BaseAdapter person2 = newPerson(sModel);
        person2.setProperty("name", "Steve");

        final BaseAdapter ref1 = newPersonReference(sModel);
        ref1.getNode().setReferent("person", person2.getNode());
        person1.addChild("friends", ref1);

        sModel.addRoot(person1);
        sModel.addRoot(person2);

        sModel.setLoading(false);
        return sModel;
    }


    public void saveModel(@org.jetbrains.annotations.NotNull SModelDescriptor sModelDescriptor) {
    }

    // =================================================================================================================

    static BaseAdapter newPerson(final SModel sm) {
        return SModelUtil_new.instantiateConceptDeclaration("myproject.structure.Person", sm, GlobalScope.getInstance(), false).getAdapter();
    }
    static BaseAdapter newPersonReference(final SModel sm) {
        return SModelUtil_new.instantiateConceptDeclaration("myproject.structure.PersonReference", sm, GlobalScope.getInstance(), false).getAdapter();
    }
}